;; brushless motor controller
;;
;; Copyright 2012 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike at mikekohn.net
;;
;; Control the speed of a brushless motor.

.include "msp430g2x31.inc"

; 2.0 ms = 152 interrupts
; 0.3 ms = 23 interrupts
; 0.6 ms = 46 interrupts

; 500ms = 38000 interrupts

; HH YAW PTCH THROTTLE YAW_CORRECT
; YAW = 0-126  63
; PITCH = 0-126 63
; THROTTLE = 0-126
; CORR = 0-126 63

RAM equ 0x0200
HEADER equ 152
SHORT equ 23
LONG equ 46

ALL_MOTORS_OFF equ 0x15

DRIVE_MOTOR_STOP equ 0x15
DRIVE_MOTOR_F_OFF equ 0xc0|0x10
DRIVE_MOTOR_B_OFF equ 0xc0|0x20
DRIVE_MOTOR_F_ON equ 0x80|0x10
DRIVE_MOTOR_B_ON equ 0x40|0x20

MOTOR_PHASE equ RAM+32
WATCHDOG equ RAM+48

START_VALUE equ 2000

;  r4 = state (0=idle, 1=header_on, 2=header_off, 3=first half, 4=second)
;  r5 = interupt count
;  r6 = pointer to next byte coming in 
;  r7 = current byte
;  r8 = bit count
;  r9 = new motor value
; r10 = interrupt count (drive)
; r11 = cycle total len
; r12 = cycle pulse len
; r13 = interrupt routine
; r14 = cycle pointer
; r15 = time before change

  .org 0xf800
start:
  ;; Turn off watchdog
  mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Please don't interrupt me
  dint

  ;; r13 points to which interrupt routine should be called
  ;mov.w #led_off, r13

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set MCLK to 16 MHz with DCO 
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

.if 0
  ;; Set MCLK to 16 MHz external crystal
  bic.w #OSCOFF, SR
  bis.b #XTS, &BCSCTL1
  mov.b #LFXT1S_3, &BCSCTL3
  ;mov.b #LFXT1S_3|XCAP_1, &BCSCTL3
test_osc:
  bic.b #OFIFG, &IFG1
  mov.w #0x00ff, r15
dec_again:
  dec r15
  jnz dec_again
  bit.b #(OFIFG), &IFG1
  jnz test_osc
  mov.b #(SELM_3|SELS), &BCSCTL2
.endif

  ;; Set up output pins
  ;; P2.7 = IR Input
  ;; P1.0 = PNP Bridge 0
  ;; P1.1 = NPN Bridge 0
  ;; P1.2 = PNP Bridge 1
  ;; P1.3 = NPN Bridge 1
  ;; P1.4 = PNP Bridge 2
  ;; P1.5 = NPN Bridge 2
  mov.b #0xff, &P1DIR
  mov.b #ALL_MOTORS_OFF, &P1OUT
  mov.b #0x00, &P2DIR
  mov.b #0x00, &P2OUT
  mov.b #0, &P2SEL

  ;; Set up Timer
  mov.w #210, &TACCR0
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  mov.b #1, &RAM    ; Yaw
  mov.b #1, &RAM+1  ; Pitch
  mov.b #1, &RAM+2  ; Throttle
  mov.b #1, &RAM+3  ; Yaw correction

  mov.w #0, &WATCHDOG
  mov.w #START_VALUE, r11
  ;mov.w #16738, r12
  mov.b #0, r7
  mov.w #0, r10

  ;; 3 cycle brushless
  mov.b #0x15, &MOTOR_PHASE
  mov.b #0x15, &MOTOR_PHASE+1
  mov.b #0x15, &MOTOR_PHASE+2

  mov.w #MOTOR_PHASE, r14

.if 0
  mov.w #0x210, r6
clear_mem:
  mov.b #0, 0(r6)
  inc r6
  cmp.w #0x250, r6
  jne clear_mem
.endif

  ;; Okay, I can be interrupted now
  eint


main:
  bit.b #0x80, &P2IN
  jeq read_command
  ;inc.w &WATCHDOG
  ;cmp.w #65535, &WATCHDOG
  ;jne main
  jmp main

  ; haven't seen a command in a while
  mov.b #0x15, &MOTOR_PHASE
  mov.b #0x15, &MOTOR_PHASE+1
  mov.b #0x15, &MOTOR_PHASE+2
  mov.w #START_VALUE, r11
  ;mov.w #16738, r12
  jmp main

read_command:
  mov.w #0, &WATCHDOG
  mov.w #0, r5
wait_low:
  bit.b #0x80, &P2IN
  jz wait_low

  ;; if r5 is less than 100, something went wrong.  Not sure how important
  ;; this check really is
  cmp.w #100, r5
  jlo main

start_signal:

  ;; signal should be low, should we count?
  ;; check there is no IR
  mov.w #0, r5
wait_ir_off_header:
  cmp.w #200, r5
  jhs main                ; pause is wayyyy too long, bail out
  bit.b #0x80, &P2IN
  jnz wait_ir_off_header  ; wait for data on IR sensor

  mov.w #RAM, r6
receive_next_byte:
  mov.b #0, r7            ; shouldn't be needed
  mov.b #8, r8
receive_next_bit:

  ;; check IR signal is on
  mov.w #0, r5
wait_ir_on:
  cmp.w #120, r5
  jhs start_signal        ; looks like the start signal
  cmp.w #70, r5
  jhs main                ; signal is wayyyy too long, bail out
  bit.b #0x80, &P2IN
  jz wait_ir_on           ; wait for data on IR sensor

  ;; check there is no IR
  mov.w #0, r5
wait_ir_off:
  cmp.w #100, r5
  jhs main                ; pause is wayyyy too long, bail out
  bit.b #0x80, &P2IN
  jnz wait_ir_off         ; wait for data on IR sensor

  rla.b r7
  cmp #SHORT+12, r5
  jlo not_a_zero
  bis.b #1, r7

not_a_zero:
  dec.b r8
  jnz receive_next_bit

  mov.b r7, 0(r6)
  inc r6

  cmp.w #RAM+4, r6
  jne receive_next_byte

  ;; We have a command now
  ;jmp main

  ;; Do drive motor
  cmp.b #61, &RAM+1
  jhs not_forward
  mov.b #011100b, &MOTOR_PHASE
  mov.b #110001b, &MOTOR_PHASE+1
  mov.b #000111b, &MOTOR_PHASE+2
  mov.b #61, r8
  sub.b &RAM+1, r8
  rla.w r8
  rla.w r8
  add.w #drive_speed, r8    ; r8 points to (speed * 4) + drive_speed
  mov.w @r8+, r9            ; r9 = speed lookup value
  ;mov.w @r8+, r12
  ;mov.w @r8+, r11
  cmp.w r9, r11             ; compare new speed with current speed
  jhs main
  mov.w r9, r11
  jmp main

not_forward:
  cmp.b #66, &RAM+1
  jhs not_stop
  mov.b #010101b, &MOTOR_PHASE
  mov.b #010101b, &MOTOR_PHASE+1
  mov.b #010101b, &MOTOR_PHASE+2
  mov.w #START_VALUE, r11
  mov.w #START_VALUE, r9
  ;mov.w #30000, r12
  jmp main

not_stop:
  mov.b #011100b, &MOTOR_PHASE
  mov.b #000111b, &MOTOR_PHASE+1
  mov.b #110001b, &MOTOR_PHASE+2
  mov.b &RAM+1, r8
  sub.b #66, r8
  rla.w r8
  rla.w r8
  add.w #drive_speed, r8
  mov.w @r8+, r9
  ;mov.w @r8+, r12
  ;mov.w @r8+, r11
  cmp.w r9, r11
  jhs main
  mov.w r9, r11
  jmp main

timer_interrupt:
  inc.w r5               ; counter for IR reader
  inc.w r10              ; counter for motor phase change

  dec.w r15              ; r15 controls the ramp-[up/down] speed
  jnz no_speed_change    ; if X interrupts didn't pass, speed can't change
  mov.w #10, r15
  cmp.w r9, r11
  jeq no_speed_change    ; if r9 == r11, speed is where we want it
  jlo slow_down
  dec.w r11              ; faster by 1 step
  jmp no_speed_change
slow_down:
  inc.w r11              ; slower by 1 step

  ;mov.w r9, r11

no_speed_change:
  cmp.w r11, r10
  jhs clear_int_count
  ;cmp.w r12, r10
  ;jlo exit_interrupt
  ;bis.b #0x15, &P1OUT      ; turn my PNP's off  (+3.3 on the gate)
exit_interrupt:
  reti 

clear_int_count:
  mov.w #0, r10
  mov.b @r14+, &P1OUT      ; next motor phase

  cmp.w #MOTOR_PHASE+3, r14
  jne motor_phase_ok
  mov.w #MOTOR_PHASE, r14
motor_phase_ok:
  reti

drive_speed:
  dw 2539, 2539, 2116, 2116, 1587, 1587, 1269, 1269, 
  dw 1058, 1058, 907, 907, 846, 846, 746, 746, 
  dw 668, 668, 604, 604, 552, 552, 529, 529, 
  dw 488, 488, 453, 453, 423, 423, 396, 396, 
  dw 384, 384, 362, 362, 343, 343, 325, 325, 
  dw 309, 309, 302, 302, 288, 288, 276, 276, 
  dw 264, 264, 253, 253, 248, 248, 239, 239, 
  dw 230, 230, 222, 222, 215, 215, 211, 211, 
  dw 204, 204, 198, 198, 192, 192, 186, 186, 
  dw 184, 184, 178, 178, 173, 173, 169, 169, 
  dw 164, 164, 162, 162, 158, 158, 154, 154, 
  dw 151, 151, 147, 147, 145, 145, 142, 142, 
  dw 139, 139, 136, 136, 133, 133, 132, 132, 
  dw 129, 129, 126, 126, 124, 124, 122, 122, 
  dw 120, 120, 118, 118, 116, 116, 114, 114, 
  dw 112, 112, 111, 111, 109, 109, 107, 107, 

  org 0xffe8
vectors:
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw timer_interrupt       ; Timer_A2 TACCR0, CCIFG
  dw 0
  dw 0
  dw 0
  dw 0
  dw 0
  dw start                 ; Reset



